#!/bin/bash
#   SPDX-License-Identifier: GPL-2.0
#   Copyright(c) 2023 Intel Corporation. All rights reserved.

# $1 is the directory containing the built RPMs
# assumes directory only contains rpms from
# rdma-core.   The directory home directory is probed
# for rdma_core_build* and the directory with the
# highest lexical sort is picked.  To override
# that selection the explicit rpm directory can be passed
# as an argument.
#
# The script assumes that ALL the rpms in install directory will be installed.
# The script also checked for packages that will be removed and replace them
# after installation.


cleanup () {
	rm -f /tmp/built$$.txt /tmp/installed$$.txt
}

trap cleanup SIGINT SIGTERM

if [ -r /etc/os-release ]
then
	source /etc/os-release
else
	echo "Unable to determine release"
	exit 1
fi
if [ -z "$1" ]
then
	# sort order determines
	builddir=$(ls -d ~/rdma_core_build* | tail -1)
	if [ -z "$builddir" -o ! -d "$builddir" ]
	then
		echo build directory cannot be determinted
		exit 1
	fi
	rpmdir=${builddir}/rpmbuild/RPMS/x86_64
else
	rpmdir=$1
	if ! ls $rpmdir/*.rpm > /dev/null 2>&1
	then
		echo "$rpmdir has no RPMs"
		exit 1
	fi
fi
if [ ! -d "${rpmdir}" ]
then
	echo ${rpmdir} not found
	exit 1
fi

case $ID in
rhel|rocky|centos|ol)
	;;
sles)
	;;
*)
	echo "Unknown distribution"
	exit 1
	;;
esac

do_install() {
	case $ID in
	rhel|rocky|centos)
		[ -f /tmp/installed$$.txt ] && rpm -e --nodeps `cat /tmp/installed$$.txt`
		if type dnf >/dev/null 2>/dev/null
		then
			dnf install -y *.rpm
		else
			yum install -y *.rpm
		fi
		;;
	sles)
		[ -f /tmp/installed$$.txt ] && rpm -e --nodeps `cat /tmp/installed$$.txt`
		if zypper install --help | grep -q -- '--allow-unsigned-rpm'
		then
			zypper install -y --allow-unsigned-rpm *.rpm
		else
			zypper install -y *.rpm
		fi
		;;
	esac
}

pushd $rpmdir
# build a list of package names from built files
rpm -qp --qf "%{NAME}\n" *.rpm > /tmp/built$$.txt
rm -f /tmp/installed$$.txt
# create a list of shortnames that are currently installed
for name in `cat /tmp/built$$.txt`
do
	if rpm -q $name 2>/dev/null
	then
		echo $name >> /tmp/installed$$.txt
	fi
done
do_install
rm -f /tmp/depraw$$.txt
cleanup
